-- By Ben Phelps - https://phelps.io/

local ItemUpgradeInfo = LibStub("LibItemUpgradeInfo-1.0")
local ScrollingTable = LibStub("ScrollingTable")

local frame = CreateFrame("Frame")
local window = CreateFrame("Frame", nil, UIParent, "BasicFrameTemplate")

local itemNames = {
	[1] = 'Head',
	[2] = 'Neck',
	[3] = 'Shoulder',
	[4] = 'Shirt',
	[5] = 'Chest',
	[6] = 'Waist',
	[7] = 'Legs',
	[8] = 'Feet',
	[9] = 'Wrist',
	[10] = 'Hands',
	[11] = 'Finger 1',
	[12] = 'Finger 2',
	[13] = 'Trinket 1',
	[14] = 'Trinket 2',
	[15] = 'Back',
	[16] = 'Main hand',
	[17] = 'Off hand',
	[18] = 'Ranged',
	[19] = 'Tabard'
}

local stf = ScrollingTable:CreateST({
		{ name = "Character", width = 150, align = "left" },
		{ name = "iLvl", width = 75, align = "right" },
		{
			name = "Last Update", width = 100, align = "right",
			comparesort = function (self, rowa, rowb, sortbycol)
				local cella, cellb = self:GetCell(rowa, sortbycol), self:GetCell(rowb, sortbycol)
				local column = self.cols[sortbycol]
				local direction = column.sort or column.defaultsort or "asc";
				if direction:lower() == "asc" then 		
					return cella.ts > cellb.ts
				else
					return cella.ts < cellb.ts
				end
    		end
    	}
}, nil, nil, {
	r = 1,
	g = 1,
	b = 1,
	a = 0.1
}, window)
stf:EnableSelection(true)

local sti = ScrollingTable:CreateST({
		{ name = "Item", width = 170, align = "left" },
		{ name = "iLvl", width = 50, align = "right" },
		{ name = "Slot", width = 65, align = "right" },
}, nil, nil, {
	r = 1,
	g = 1,
	b = 1,
	a = 0.1
}, window)

window:SetMovable(true)
window:EnableMouse(true)
window:RegisterForDrag("LeftButton")
window:SetScript("OnDragStart", frame.StartMoving)
window:SetScript("OnDragStop", frame.StopMovingOrSizing)
window:SetWidth(350)
window:SetHeight(150)
window:SetPoint("CENTER", UIParent, CENTER)
window.TitleText:SetText("Raid Counsel")
window.open = false
window.CloseButton:SetScript("OnClick", function(self)
	window.open = false
	HideParentPanel(self)
end)
window:Hide()

filterRaidButton = CreateFrame("CheckButton", "RaidFilterRaidButton", window, "UICheckButtonTemplate")
filterRaidButton:SetPoint("BOTTOMLEFT", window, "BOTTOMLEFT", 175, 1)
filterRaidButton.text = _G["RaidFilterRaidButtonText"]
filterRaidButton.text:SetText("Only Raid")


local filterName = CreateFrame("EditBox", nil, window, "SearchBoxTemplate")
filterName:SetWidth(160)
filterName:SetHeight(20)
filterName:SetPoint("BOTTOMLEFT", window, "BOTTOMLEFT", 15, 7)


local function filterRaid(self, row)
	if row.cols then
		local name = row.cols[1].value
		if filterRaidButton:GetChecked() then
			local nRaidMembers = GetNumGroupMembers()
			local found = false
			for i = 1, nRaidMembers do
				local raidMember = GetRaidRosterInfo(i)
				if name == raidMember then
					found = true
				end
			end
			if not found then return false end
		end
		if filterName:GetText() ~= '' then
			local search, _ = string.find(strlower(name), strlower(filterName:GetText()))
			if search ~= nil then
				return true
			else
				return false
			end
		else
			return true
		end
	end
	return false
end
filterName:SetScript("OnTextChanged", function(self, userInput)
	stf:SetFilter(filterRaid)
	SearchBoxTemplate_OnTextChanged(self)
end)
filterRaidButton:SetScript("PostClick", function(self, event, arg1)
	stf:SetFilter(filterRaid)
end)
stf:SetFilter(filterRaid)

local inspectButton = CreateFrame("Button", nil, window, "SecureActionButtonTemplate,UIPanelButtonTemplate")
inspectButton:SetPoint("BOTTOMRIGHT", window, "BOTTOMRIGHT", -6, 6)
inspectButton:SetWidth(130)
inspectButton:SetHeight(24)
inspectButton:SetText("Inpsect Target")
inspectButton:SetAttribute("type", "macro")
inspectButton:SetAttribute("macrotext", "/inspect")
inspectButton:SetScript("PostClick", function(self)
	self:Disable()
	self:SetText("Inspecting...")
	C_Timer.NewTicker(1.5, function()
		self:SetText("Inpsect Target")
		self:Enable()
	end, 1)
end)
inspectButton:Show()

stf:RegisterEvents({
    ["OnClick"] = function (rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable, ...)
    	if not realrow then return false end
        local rowdata = data[realrow]
    	local celldata = data[realrow].cols[1]
    	local gear = RaidCounsel_Data.characters[celldata.value].gear
    	if gear then
    		sti:SetData(gear, true)
    	else
    		sti:SetData({{'N/A', 'N/A', 'N/A'}}, true)
    	end
    	return false
    end,
})

sti:RegisterEvents({
    OnEnter = function (rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable, ...)
    	if not realrow then return false end
        local rowdata = data[realrow]
    	local celldata = data[realrow]
    	if celldata[1] then
    		local item = celldata[1]
	    	if item then
	    		GameTooltip:SetOwner(cellFrame, "ANCHOR_CURSOR")
				GameTooltip:SetHyperlink(item)
				GameTooltip:Show()
	    	end
    	end
    	return false
    end,
    OnLeave = function()
    	GameTooltip:Hide()
	end
})

local function round(num, idp)
	local mult = 10^(idp or 0)
	return math.floor(num * mult + 0.5) / mult
end

local st = { }
local function createST()
	table.wipe(st)
	for name, info in pairs(RaidCounsel_Data.characters) do
		local color = RAID_CLASS_COLORS[info.class]
		local row = {
			cols = {
				{
					value = name,
					color = {
						r = color.r,
						g = color.g,
						b = color.b,
						a = color.a
					}
				},
				{ value = info.ilvl },
				{ value = timeago(info.updated), ts = info.updated }
			},
		}
		table.insert(st, row)
	end
	return st
end

local function displayTable()
	if not window.open then
		stf:SetData(createST())
		tableShown = true
		stf.frame:SetPoint("TOPLEFT", window, "TOPLEFT", 4, -42)
		sti.frame:SetPoint("TOPRIGHT", window, "TOPRIGHT", -4, -42)
		window:SetWidth(stf.frame:GetWidth()+5+sti.frame:GetWidth())
		window:SetHeight(stf.frame:GetHeight()+42+5+26)
		window:Show()
		window.open = true
	end
end

function timeago(t)

  local span = time() - t

  num = span
  if num < 45 then return "just now" end

  num = span / 60
  if num < 1.5 then return "1 " .. "minute ago" end
  if num < 59.5 then return round(num) .. " " .. "minutes ago" end

  num = span / 3600
  if num < 1.5 then return "1 " .. "hour ago" end
  if num < 23.5 then return round(num) .. " " .. "hours ago" end

  num = span / 86400
  if num < 1.5 then return "1 " .. "day ago" end
  if num < 7.5 then return round(num) .. " " .. "days ago" end

  if num / 7 < 1.5 then return "1 " .. "week ago" end
  if num / 7 < 4.5 then return round(num / 7) .. " " .. "weeks ago" end

  if num / 30 < 1.5 then return "1 " .. "month ago" end
  if num / 30 < 11.5 then return round(num / 30) .. " " .. "months ago" end

  if num / 365.25 < 1.5 then return "1 " .. "year ago" end
  return round(num / 365.25) .. " " .. "years ago"
end

function GetAvgIlevel(unit)
	local ilvl = 0
	local total = 0
	local items = 0
	if InspectFrame:IsShown() then
		if UnitExists(unit) then
			local name = GetUnitName(unit, true)
			if not RaidCounsel_Data.characters[name] then
				local _, class = UnitClass("target")
				RaidCounsel_Data.characters[name] = {
					class = class,
					updated = time(),
					gear = { }
				}
			else
				RaidCounsel_Data.characters[name].gear = { }
				RaidCounsel_Data.characters[name].updated = time()
			end
			for i=INVSLOT_FIRST_EQUIPPED, INVSLOT_LAST_EQUIPPED  do
				local link = GetInventoryItemLink(unit, i)
				if link then
					local ilvl = ItemUpgradeInfo:GetUpgradedItemLevel(link)
					if ilvl > 1 then
						total = total + ilvl
						items = items + 1
					end
					local item = {
						[1] = link,
						[2] = ilvl,
						[3] = itemNames[i],
					}
					table.insert(RaidCounsel_Data.characters[name].gear, item)
				end
			end
			ilvl = round(total / items, 1)
			RaidCounsel_Data.characters[name].ilvl = ilvl
			stf:SetData(createST())
			sti:SetData(RaidCounsel_Data.characters[name].gear, true)
		end
	end
	return ilvl
end

local updating
local function handle(self, event, ...)
	if event == 'INSPECT_READY' then
		if not updating then
			if InspectFrame.unit then
				updating = true
				C_Timer.NewTicker(0.4, function()
					GetAvgIlevel(InspectFrame.unit)
					stf:SetData(createST())
					updating = false
					inspectButton:SetText("Inpsect Target")
					inspectButton:Enable()
				end, 1)
			end
		end
	elseif event == 'ADDON_LOADED' then
		if RaidCounsel_Data == nil then
			RaidCounsel_Data = {
				characters = { }
			}
		end
	end
end

frame:RegisterEvent("INSPECT_READY")
frame:RegisterEvent("ADDON_LOADED")
frame:SetScript("OnEvent", handle)

SLASH_RAIDCOUNSEL1, SLASH_RAIDCOUNSEL2 = "/raidcounsel", "/rc"
SlashCmdList["RAIDCOUNSEL"] = function(msg, editBox)
	displayTable()
end
